﻿using System;
using System.Windows.Forms;
using System.Collections.Generic;
using System.ServiceModel;

namespace EmployeeManagement
{
    //GUIクラス。
    public partial class MainForm : Form
    {
        ManagementSystem system = new ManagementSystem();

        public MainForm()
        {
            InitializeComponent();
        }

        private void buttonAdd_Click(object sender, EventArgs e)
        {
            if (system.Add(textBoxName.Text, textBoxAge.Text, textBoxPosition.Text))
            {
                textBoxName.Text = textBoxAge.Text = textBoxPosition.Text = string.Empty;
            }
        }

        private void buttonFind_Click(object sender, EventArgs e)
        {
            EmployeeData hit = system.Find(textBoxName.Text);

            //GUIに表示
            if (hit != null)
            {
                textBoxName.Text = hit.Name;
                textBoxAge.Text = hit.Age.ToString();
                textBoxPosition.Text = hit.Post;
            }
            else
            {
                textBoxName.Text = textBoxAge.Text = textBoxPosition.Text = string.Empty;
            }
        }

        ServiceHost wcfHost = null;
        public const int WM_OPEN_SERVER = 0x8000;
        public const string WcfAddresses = "net.pipe://localhost/EmployeeManagement.ManagementSystem";

        protected override void WndProc(ref Message m)
        {
            if (m.Msg == WM_OPEN_SERVER)
            {
                wcfHost = new ServiceHost(system);
                wcfHost.AddServiceEndpoint(typeof(IManagementSystem),
                    new NetNamedPipeBinding(), WcfAddresses);
                wcfHost.Open();
                m.Result = new IntPtr(1);
            }
            base.WndProc(ref m);
        }

        protected override void OnClosed(EventArgs e)
        {
            if (wcfHost != null)
            {
                wcfHost.Close();
                wcfHost = null;
            }
            base.OnClosed(e);
        }
    }

    //社員データ
    [Serializable]
    public class EmployeeData
    {
        public string Name { get; set; }
        public int Age { get; set; }
        public string Post { get; set; }

        public EmployeeData(string name, int age, string post)
        {
            Name = name;
            Age = age;
            Post = post;
        }
    }

    //管理部分。
    [ServiceBehavior(InstanceContextMode = InstanceContextMode.Single)]
    public class ManagementSystem : IManagementSystem
    {
        List<EmployeeData> allData = new List<EmployeeData>();

        public bool Add(string name, string ageString, string post)
        {
            int age;
            if (!int.TryParse(ageString, out age))
            {
                return false;
            }
            if (Find(name) != null)
            {
                return false;
            }
            allData.Add(new EmployeeData(name, age, post));
            return true;
        }

        public EmployeeData Find(string name)
        {
            foreach (EmployeeData element in allData)
            {
                if (element.Name == name)
                {
                    return element;
                }
            }
            return null;
        }
    }

    [ServiceContract]
    public interface IManagementSystem
    {
        [OperationContract]
        bool Add(string name, string ageString, string post);

        [OperationContract]
        EmployeeData Find(string name);
    }
}
