﻿using System;
using EmployeeManagement;
using System.Diagnostics;
using System.IO;
using System.Runtime.InteropServices;
using System.ServiceModel;
using System.Threading;

namespace EmployeeManagementAPI
{
    public class EmployeeManagementApp : IDisposable
    {
        //開始。
        public static EmployeeManagementApp Start()
        {
#if DEBUG
            string path = Path.GetFullPath("../../../bin/Debug/EmployeeManagement.exe");
#else
            string path = Path.GetFullPath("../../../bin/Release/EmployeeManagement.exe");
#endif
            return new EmployeeManagementApp(Process.Start(path));
        }

        IManagementSystem managementSystem;
        int processId;

        [DllImport("user32.dll", CharSet = CharSet.Auto)]
        static extern IntPtr SendMessage(IntPtr hWnd, int msg, IntPtr wParam, IntPtr lParam);
        
        //コンストラクタ。
        EmployeeManagementApp(Process process)
        {
            processId = process.Id;

            //メインウィンドウが完成するまで待つ。
            while (process.MainWindowHandle == IntPtr.Zero)
            {
                process = Process.GetProcessById(process.Id);
                Thread.Sleep(10);
            }

            //インターフェイス公開要求。
            SendMessage(process.MainWindowHandle, MainForm.WM_OPEN_SERVER, IntPtr.Zero, IntPtr.Zero);

            //インターフェイス呼び出し。
            managementSystem = new ChannelFactory<IManagementSystem>(new NetNamedPipeBinding(), MainForm.WcfAddresses).CreateChannel();
        }

        //終了。
        public void Dispose()
        {
            Process.GetProcessById(processId).CloseMainWindow();
        }

        //登録。
        public bool Add(string name, string age, string post)
        {
            return managementSystem.Add(name, age, post);
        }

        //検索。
        public EmployeeData Find(string name)
        {
            return (EmployeeData)managementSystem.Find(name);
        }
    }
}
